class AspRpt::TemplatesController < ApplicationController

  before_filter :major_tab_rpt_requests
  before_filter :admin_filter

  SKIP_HTML_SANITIZATION = ["sql_text"]

  def index
    @reports = AspRpt::Template.built_in + AspRpt::Template.user_created
    if params[:table_list_sort] && params[:table_list_order]
      if params[:table_list_sort] == 'name'
        if params[:table_list_order] == 'a'
          @reports.sort! { |x,y| x.name.downcase <=> y.name.downcase }
        else
          @reports.sort! { |x,y| y.name.downcase <=> x.name.downcase }
        end
      end
      if params[:table_list_sort] == 'created_by' || params[:table_list_sort] == 'type'
        if params[:table_list_order] == 'a'
          @reports.sort! { |x,y| x.owner_id.to_i <=> y.owner_id.to_i }
        else
          @reports.sort! { |x,y| y.owner_id.to_i <=> x.owner_id.to_i }
        end
      end
      if params[:table_list_sort] == 'category'
        if params[:table_list_order] == 'a'
          @reports.sort! { |x,y| x.report_class.downcase <=> y.report_class.downcase }
        else
          @reports.sort! { |x,y| y.report_class.downcase <=> x.report_class.downcase }
        end
      end
    end
  end

  def new
    @rpt_template = AspRpt::Template.new
  end

  def createbasic
    @rpt_template = AspRpt::TemplateBasic.new #(params[:asp_rpt_template_basic])

    set_from_params

    if @rpt_template.save
      if params[:commit] == 'Create and run'
        redirect_to :action => :new, :controller => 'requests', :rpt_template_id => @rpt_template.id and return false
      else
        flash[:notice] = "Report type successfully created."
        redirect_to(rpt_templates_url) and return false
      end
    else
      prep_basic_template_vars
      render :action => 'newbasic'
    end
  end

  def newbasic
    @rpt_template = AspRpt::TemplateBasic.new
    prep_basic_template_vars
  end

  def editbasic
    @rpt_template = AspRpt::TemplateBasic.find(params[:id])
    unless @rpt_template.basic?
      raise 'Attempt to use "Edit Basic" with a template that is not basic.'
    end

    if @rpt_template.built_in?
      flash[:error]='Cannot edit built-in reports'
      redirect_to :action => :index and return
    end
    prep_basic_template_vars
  end

  def updatebasic
    if params[:commit] == 'Cancel'
      flash[:notice] = 'Edit cancelled'
      redirect_to :action => :index and return
    end

    successful = false
    @rpt_template = AspRpt::TemplateBasic.find(params[:id])

    unless @rpt_template.basic?
      raise 'Attempt to use "Update Basic" with a template that is not basic.'
    end

    if @rpt_template.built_in?
      flash[:error]='Cannot edit built-in reports'
      redirect_to :action => :index and return
    end

    @pending_requests = @rpt_template.requests.pending
    if @pending_requests.any? && !params[:confirm]
      # create a new request object to use to populate the confirm form's hidden controls
      @rpt_template = AspRpt::TemplateBasic.new #(params[:asp_rpt_template_basic])
      set_from_params
      render :action => "confirm_update_basic" and return
    end

    ActiveRecord::Base.transaction do
      set_from_params
      @rpt_template.updated_at = Time.zone.now
      successful = @rpt_template.save
      raise ActiveRecord::Rollback unless successful
    end

    if successful
      @pending_requests.each {|p|
        p.status = Console::STATUS_CANCELLED
        p.error_text = 'Administrator has modified this report type. Please run a new instance of this report.'
        p.stopped_at = Time.zone.now
        p.save_with_validation(false)
      }
      
      if params[:commit] == 'Save and run'
        redirect_to :action => :new, :controller => 'requests', :rpt_template_id => params[:id]
      else
        flash[:notice] = 'Report type was successfully updated.'
        redirect_to(rpt_templates_url) and return false
      end
    else
      prep_basic_template_vars
      render :action => "editbasic"
    end
  end

  def create
    @rpt_template = AspRpt::Template.new(params[:rpt_template])
    @rpt_template.owner = current_user
    @rpt_template.report_class = 'Advanced'

    if @rpt_template.save
      if params[:commit] == 'Create and run'
        redirect_to :action => :new, :controller => 'requests', :rpt_template_id => @rpt_template.id
      else
        flash[:notice] = 'Report type was successfully created.'
        redirect_to_templates_or_params
      end
    else
      render :action => "new"
    end
  end

  def edit    
    @rpt_template = AspRpt::Template.find(params[:id])
    unless @rpt_template.advanced?
      raise 'Attempt to use "Edit Advanced" with a template that is not advanced.'
    end
    
    if @rpt_template.built_in?
      flash[:error]='Cannot edit built-in reports'
      redirect_to :action => :index and return
    end
  end

  def update
    if params[:commit] == 'Cancel'
      flash[:notice] = 'Edit cancelled'
      redirect_to :action => :index and return
    end
      
    @rpt_template = AspRpt::Template.find(params[:id])
    unless @rpt_template.advanced?
      raise 'Attempt to use "Update Advanced" with a template that is not advanced.'
    end
        
    if @rpt_template.built_in?
      flash[:error]='Cannot edit built-in reports'
      redirect_to :action => :index and return
    end
    @rpt_template.attributes = params[:rpt_template]     
    @pending_requests = @rpt_template.requests.pending
    if @pending_requests.any? && !params[:confirm]
      render :action => "confirm_update" and return
    end

    @rpt_template.owner = current_user

    if @rpt_template.save
      @pending_requests.each {|p|
        p.status = Console::STATUS_CANCELLED
        p.error_text = 'Administrator has modified this report type. Please run a new instance of this report.'
        p.stopped_at = Time.zone.now
        p.save_with_validation(false)
      }
      @rpt_template.updated_at = Time.zone.now
      if params[:commit] == 'Save and run'
        redirect_to :action => :new, :controller => 'requests', :rpt_template_id => params[:id]
      else
        flash[:notice] = 'Report type was successfully updated.'
        redirect_to_templates_or_params
      end
    else
      render :action => "edit"
    end
  end

  def destroy
    @rpt_template = AspRpt::Template.find(params[:id])
    if @rpt_template.built_in?
      flash[:error]='Cannot delete built-in reports'
      redirect_to :action => :index and return
    end
    @rpt_template.destroy
    flash[:notice] = 'Report type deleted.'
    redirect_to(rpt_templates_url)
  end

  def copy
    original_template = AspRpt::Template.find(params[:id])
    @rpt_template = original_template.copy
    @rpt_template.owner = current_user
    @rpt_template.save!
    flash[:notice] = "Report type copied"
    if original_template.report_class == 'Basic'
      redirect_to editbasic_rpt_template_path(@rpt_template)
    else
      redirect_to edit_rpt_template_path(@rpt_template)
    end
  end

  def org_choice_change
    @already_chosen = []
    params.keys.sort.each do |key|
      if /^oc([0-9])+$/ =~ key
        @already_chosen << params[key]
      end
    end
    # prepare a fake rpt_template object to hang org choices on
    @rpt_template = AspRpt::TemplateBasic.new
    @already_chosen.each_with_index { |org_choice, i|
      @rpt_template.org_choices << AspRpt::TemplateBasicOrgChoice.new({:name => org_choice, :position => i})
    }
    
    prepare_applicable_columns(@already_chosen)
  end

  private

  def set_from_params(destroy_existing_children = false)
    @rpt_template.owner = current_user
    @rpt_template.report_class = 'Basic'
    @rpt_template.name = params[:asp_rpt_template_basic][:name]
    @rpt_template.description = params[:asp_rpt_template_basic][:description]
    @rpt_template.dont_use_file_data = params[:asp_rpt_template_basic][:dont_use_file_data]

    # remove existing columns and org choices
    if @rpt_template.id
      if destroy_existing_children
        @rpt_template.columns.destroy_all
        @rpt_template.filters.destroy_all
        @rpt_template.org_choices.destroy_all
        @rpt_template.sort_columns.destroy_all
      else
        @rpt_template.columns = []
        @rpt_template.filters = []
        @rpt_template.org_choices = []
        @rpt_template.sort_columns = []
      end
    end

    if params[:asp_rpt_template_basic][:org_choices]
      org_choice_list = params[:asp_rpt_template_basic][:org_choices].collect { |oc|
        AspRpt::TemplateBasicOrgChoice.new(
          :name => oc[1][:name],
          :position => oc[1][:position]
        )
      }.sort { |a, b| a.position <=> b.position }
      AspRpt::OrgChoice.assign_report_groups!(org_choice_list)
      org_choice_list.each { |new_oc|
        @rpt_template.org_choices << new_oc
      }
    end

    if params[:asp_rpt_template_basic][:columns]
      params[:asp_rpt_template_basic][:columns].each do |col|
        new_col = AspRpt::TemplateBasicColumn.new
        new_col.name = col[1][:name]
        new_col.position = col[1][:position]
        @rpt_template.columns << new_col
      end
    end

    if params[:asp_rpt_template_basic][:sorts]
      params[:asp_rpt_template_basic][:sorts].each do |sort|
        new_sort = AspRpt::TemplateBasicSort.new
        new_sort.ascending = sort[1][:ascending]
        new_sort.name = sort[1][:name]
        new_sort.org_choice_name = sort[1][:org_choice_name]
        new_sort.position = sort[1][:position]
        @rpt_template.sort_columns << new_sort
      end
    end

    if params[:asp_rpt_template_basic][:filters]
      params[:asp_rpt_template_basic][:filters].each do |filter|
        new_filter = AspRpt::TemplateBasicFilter.new
        new_filter.expression_name = filter[1][:expression_name]
        new_filter.negate = filter[1][:negate]=="1"
        new_filter.operator_name = filter[1][:operator_name]
        new_filter.value = filter[1][:value]
        @rpt_template.filters << new_filter
      end
    end

  end

  def prepare_applicable_columns(selected_org_choices)
    @choice_applicable_columns = {}
    @choice_default_sort = {}

    AspRpt::OrgChoice.assign_report_groups!(@rpt_template.org_choices)
    groups = @rpt_template.org_choices.collect { |oc| oc.group }

    groups.each_with_index { |group, index|
      @choice_applicable_columns[selected_org_choices[index]] = AspRpt::TemplateBasicColumn.applicable_for_group(group)
      @choice_default_sort[selected_org_choices[index]] = group.default_sort
    }
    applicable_columns = @choice_applicable_columns.collect { |choicename, expression| expression }.flatten.sort { |a, b| a.description <=> b.description }
    @applicable_columns_json = applicable_columns.collect { |c|
      {
        :name => c.name,
        :description_with_custom =>  ERB::Util.json_escape(c.description_with_custom),
        #        :data_type => c.data_type.name,
        :help_text => c.help_text,
        :report_column_type => c.report_column_type,
        }
    }.unique.to_json
    @applicable_column_names_json = applicable_columns.collect(& :name).to_json

    @choice_default_sort_json = {}
    @choice_default_sort.each { |choicename, sort|
      @choice_default_sort_json[choicename] = ERB::Util.json_escape(AspExp::Expression[sort].description_with_custom)
    }
    @choice_default_sort_json = @choice_default_sort_json.to_json

    @choice_applicable_columns_json = {}
    @choice_applicable_columns.each { |choicename, columns|
      @choice_applicable_columns_json[choicename] = columns.collect { |c| c.name.to_s }.sort
    }
    @choice_applicable_columns_json = @choice_applicable_columns_json.to_json
  end

  def redirect_to_templates_or_params
    url = @rpt_template.params.present? ? rpt_template_params_url(@rpt_template) : rpt_templates_url
    redirect_to url
  end

  def prep_basic_template_vars
    selected_org_choices = @rpt_template.org_choices.collect { |oc| oc.name }
    prepare_applicable_columns(selected_org_choices)

    @org_choices_json = {}
    AspRpt::OrgChoice.all.each { |oc|
      @org_choices_json[oc.name] = {:description => ERB::Util.json_escape(oc.description)}
    }
    @org_choices_json = @org_choices_json.to_json
    @filter_fields = AspExp::Expression.find(:filter => true).sort { |a, b| a.description_with_custom.downcase <=> b.description_with_custom.downcase }
    @filter_fields_json = {}
    @filter_fields.each { |f|
      filter_ops = f.data_type.filter_ops.collect { |o|
        {
          :name => o.name,
          :description => o.description(f.data_type)
        }
      }
      @filter_fields_json[f.name] = {
        :description_with_custom => ERB::Util.json_escape(f.description_with_custom),
        :filter_ops => filter_ops
      }
    }
    @filter_fields_json = @filter_fields_json.to_json

    @filter_ops_json = {}
    AspExp::Operator.comparators.each { |c|
      @filter_ops_json[c.name] = {
        :unary => c.unary?
      }
    }
    @filter_ops_json = @filter_ops_json.to_json

    @columns = AspRpt::TemplateBasicColumn.all_report_columns
    column_hash = {}
    @columns.each { |c|
      column_hash[c.name] = {
        :description_with_custom => ERB::Util.json_escape(c.description_with_custom),
        :help_text => c.help_text || '',
        }
    }
    @columns_json = column_hash.to_json
  end

end
