module NotifierHelper

  def h_description
    try {@job_step.job.description}
  end

  def h_contact
    try {@job.contact}
  end
  
  def h_started_at
    try {
      time = @job_step.job.sync? ?  @job_step.job.sync_started_at : @job_step.started_at
      time_in_zone(time)
    }
  end
  
  def h_stopped_at
    try { time_in_zone(@job_step.ended_at) }
  end
  
  def time_in_zone(time)
    tiz = time.in_time_zone(@time_zone).to_s[0, 19]
    "#{tiz} #{@time_zone}"
  end
  
  def h_source_node
    try {node_desc(@job_step.source_node_name, @job_step.source_node_address)}
  end

  def h_source_endpoint
    try {(@job_step.source_login && @job_step.source_login.name) || "#{@job_step.source_ssh_login}@#{@job_step.source_node_address}"}
  end

  def h_all_source_paths
    try {@job_step.all_source_paths.join("\n")}
  end
  
  def h_html_all_source_paths
    try {plain_to_html(h_all_source_paths)}
  end
  
  def h_first_5_source_paths
    try {@job_step.all_source_paths[0, 5].join("\n")}
  end
  
  def h_html_first_5_source_paths
    try { plain_to_html(h_first_5_source_paths) }
  end
  
  def h_destination_node
    try {node_desc(@job_step.dest_node_name, @job_step.dest_node_address)}
  end

  def h_destination_endpoint
    try {(@job_step.dest_login && @job_step.dest_login.name) || "#{@job_step.dest_ssh_login}@#{@job_step.dest_node_address}"}
  end

  def h_destination_path
    try {@job_step.dest_path}
  end
  
  def h_comment
    try {@job_step.job.comment}
  end

  def h_session_id
    try {@job_step.sessions.last.session_id}
  end

  def h_bytes_transferred
    if @job_step.job.sync?
      try { gr_size(@job_step.job.sync_total_bytes_transferred) }
    else
      try { gr_size(@job_step.total_bytes_transferred) }
    end
  end

  def h_expected_total_bytes
    try { @job_step.total_size == "unknown" ? "unknown" : gr_size(@job_step.total_size) }
  end

  def h_elapsed_time
    if @job_step.job.sync?
      try {gr_elapsed @job_step.job.sync_total_time}
    else
      try {gr_elapsed(@job_step.ended_at - @job_step.started_at) }
    end
  end
  
  def h_html_elapsed_time
    h_elapsed_time
  end
  
  def h_average_rate
    if @job_step.job.sync?
      try { gr_rate @job_step.job.sync_average_rate }
    else
      try { gr_rate @job_step.average_rate }
    end
  end

  def h_files_transferred
    if @job_step.job.sync?    
      try { gr_commify(@job_step.job.sync_total_files_complete) }
    else
      try { gr_commify(@job_step.total_files_complete) }
    end
  end

  def h_expected_total_files
    try { gr_commify(@job_step.files_pretransfer) }
  end

  def h_source_failover_group
    try {
      node = FaspNode.find_by_id(@job_step.source_node_id)
      node.nil? ? '' : node.node_group_name
    }
  end

  def h_dest_failover_group
    try {
      node = FaspNode.find_by_id(@job_step.dest_node_id)
      node.nil? ? '' : node.node_group_name
    }
  end

  def h_files_skipped
    if @job_step.job.sync?
      try { gr_commify(@job_step.job.sync_total_files_skipped) }
    else
      try { gr_commify(@job_step.last_files_skipped) }
    end
  end
  
  def h_files_failed
    if @job_step.job.sync?
      try { gr_commify(@job_step.job.sync_total_files_failed) }
    else
      try { gr_commify(@job_step.total_files_failed) }
    end
  end

  def h_first_10_filepaths
    if @job_step.job.sync?
      try { URI.decode(@job_step.job.sync_files(10).map(&:file_fullpath).join("\n")) }
    else
      try { URI.decode(@job_step.files(10).map(&:file_fullpath).join("\n")) }
    end
  end

  def h_html_first_10_filepaths
    try { plain_to_html(h_first_10_filepaths) }
  end

  def h_first_10_filenames
    if @job_step.job.sync?
      try { @job_step.job.sync_files(10).map(&:file_basename).join("\n") }
    else
      try { @job_step.files(10).map(&:file_basename).join("\n") }
    end
  end

  def h_html_first_10_filenames
    try { plain_to_html(h_first_10_filenames) }
  end

  # not truly 'all', apply a max from config for safety
  def h_all_filenames
    try {
      files = @job_step.job.sync? ? @job_step.job.sync_files(Console.email_file_list_max_length) : @job_step.files(Console.email_file_list_max_length)
      files.map(&:file_basename).join("\n")
    }
  end

  def h_html_all_filenames
    try { plain_to_html(h_all_filenames) }
  end

  # not truly 'all', apply a max from config for safety
  def h_all_filepaths
    try {
      files = @job_step.job.sync? ? @job_step.job.sync_files(Console.email_file_list_max_length) : @job_step.files(Console.email_file_list_max_length)
      files.map(&:file_fullpath).join("\n")
    }
  end
  
  def h_html_all_filepaths
    try { plain_to_html(h_all_filepaths) }
  end

  def h_first_10_filepaths_w_checksums
    if @job_step.job.sync?
      try { URI.decode(@job_step.job.sync_files(10).map(&:file_fullpath_w_checksum).join("\n")) }
    else
      try { URI.decode(@job_step.files(10).map(&:file_fullpath_w_checksum).join("\n")) }
    end
  end

  def h_html_first_10_filepaths_w_checksums
    try { plain_to_html(h_first_10_filepaths_w_checksums) }
  end

  # not truly 'all', apply a max from config for safety
  def h_all_filepaths_w_checksums
    try {
      files = @job_step.job.sync? ? @job_step.job.sync_files(Console.email_file_list_max_length) : @job_step.files(Console.email_file_list_max_length)
      files.map(&:file_fullpath_w_checksum).join("\n")
    }
  end

  def h_html_all_filepaths_w_checksums
    try { plain_to_html(h_all_filepaths_w_checksums) }
  end

  def h_error_message
    try {@job_step.err_desc}
  end

  def h_last_session_id
    try { @job_step.sessions.last(:order => 'id').session_id }
  end

  # append (node.ip.address) if name is not already the ip address
  def node_desc(node_name, node_address)
    node_name == node_address ? node_name : "#{node_name} (#{node_address})"
  end

  # Take string delimited by newline, and return html-escaped string delmited by <br />
  def plain_to_html(string)
    string.split("\n").
        map { |e| h(e) }.
        join("<br />")
  end
  
  private
  
  # Plain text templates call h_foo methods, html templates call h_html_foo.
  # Use method missing to delegate html calls to plain text calls.
  #
  # Create a h_html_foo method to have the html template use a different
  # method than the plain template.
  def method_missing(method, *args, &block)
    if method.to_s =~ /^h_html_(.*)$/
      h(send("h_#{$1}", *args, &block))
    else
      super
    end
  end
  
end