# This class is for collecting and validating watchfolder parameters, and producing
# JSON requests for the watchfolder API
class AsperaWatchfolderValidation < ActiveRecord::BaseWithoutTable
  column :name, :string

  column :local_node, :integer
  column :local_path, :string


  column :remote_authentication_method, :string
  column :remote_ssh_key_id, :integer

  column :remote_password, :string
  column :local_ssh_key_path, :string
  column :token, :string
  column :proxy, :string

  column :remote_address, :string
  column :remote_login, :string
  column :remote_tcp_port, :integer
  column :remote_nodeapi_port, :integer
  column :remote_udp_port, :integer
  column :remote_path, :string

  column :target_rate, :integer
  column :min_rate, :integer
  column :policy, :string
  column :max_retries, :integer
  column :retry_timeout, :integer
  column :retry_period, :integer
  column :cipher, :string

  column :snapshot_creation_period, :integer
  column :queue_threshold, :integer
  column :connect_timeout, :integer
  column :sample_period, :integer

  column :cool_off, :integer

  column :resume, :boolean
  column :resume_selection, :string
  column :overwrite, :string
  column :preserve_uid, :boolean
  column :preserve_gid, :boolean
  column :preserve_time, :boolean
  column :preserve_creation_time, :boolean
  column :preserve_modification_time, :boolean
  column :preserve_access_time, :boolean

  column :temp_dir, :string
  column :delete_after_transfer_per_drop, :boolean
  column :delete_after_transfer_per_file, :boolean
  column :archive_after_transfer, :boolean
  column :archive_dir, :string

  # EAR Settings
  column :enable_content_protection, :boolean
  column :content_protect_password, :string
  column :content_protect_password_confirmation, :string

  # Drops
  column :drop_detection_strategy, :string
  column :drop_detection_cool_off, :integer

  # Growing Files
  column :grow_max_parallel, :integer
  column :grow_target_rate, :integer
  column :grow_policy, :string
  column :grow_cipher, :string
  column :grow_tcp_port, :integer
  column :grow_udp_port, :integer
  column :grow_completion_timeout, :integer
  column :grow_memory, :integer
  column :grow_chunk_size, :integer

  # Package
  column :package_timeout, :integer
  column :package_use_filter, :boolean
  column :package_final_transfer, :string

  # Unused in UI, but necessary for full request for updates
  column :version, :integer
  column :fingerprint, :string
  column :keypath, :string
  column :uuid, :string
  column :mode, :string
  column :order, :string
  column :local_watchfolder_type, :string
  column :disable_watchfolder_type, :boolean
  column :local_node_pull, :integer

  column :cookie, :string
  column :datagram_size, :integer
  column :grow_connect_timeout, :integer
  column :grow_datagram_size, :integer
  column :grow_force_send_after, :integer
  column :grow_min_rate, :integer
  column :grow_overwrite, :string
  column :grow_preserve_access_time, :boolean
  column :grow_preserve_creation_time, :boolean
  column :grow_preserve_gid, :boolean
  column :grow_preserve_modification_time, :boolean
  column :grow_preserve_time, :boolean
  column :grow_preserve_uid, :boolean
  column :grow_read_block_size, :integer
  column :grow_resume, :boolean
  column :grow_resume_selection, :string
  column :grow_rexmsg_size, :integer
  column :grow_write_block_size, :integer
  column :max_parallel, :integer
  column :raw_options, :string
  column :read_block_size, :integer
  column :rexmsg_size, :integer
  column :symbolic_links, :string
  column :synchronize_empty_directory, :boolean
  column :tags, :string
  column :watchd_connection, :string
  column :watchd_expire_in, :integer
  column :watchd_identifier, :string
  column :watchd_scan_period, :integer
  column :watchd_uri, :string
  column :write_block_size, :integer

  # Shouldn't require new password when editing
  column :require_password

  column :watchfolder_filters, :string
  column :growing_file_filters, :string
  column :package_filters, :string

  serialize :watchfolder_filters, Array
  serialize :growing_file_filters, Array
  serialize :package_filters, Array

  validates_presence_of :name
  validates_presence_of :local_node
  validates_presence_of :local_path
  validates_presence_of :remote_address
  validates_presence_of :remote_login
  validate              :remote_credentials_present?, :if => :require_password
  validates_presence_of :remote_path

  validates_length_of :name, :maximum => 255, :message => "^Watchfolder name is too long (maximum is 255 characters)"

  validates_numericality_of :remote_tcp_port, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 65536
  validates_numericality_of :remote_udp_port, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 65536
  validates_numericality_of :target_rate, :allow_blank => true
  validates_numericality_of :min_rate, :allow_blank => true
  validates_numericality_of :max_retries, :allow_blank => true
  validates_numericality_of :retry_timeout, :allow_blank => true
  validates_numericality_of :retry_period, :allow_blank => true
  validates_numericality_of :snapshot_creation_period, :allow_blank => true, :greater_than_or_equal_to => 0, :less_than_or_equal_to => 10000000

  validates_numericality_of :cool_off, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000
  validates_numericality_of :queue_threshold, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000
  validates_numericality_of :connect_timeout, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000
  validates_numericality_of :sample_period, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000

  validates_numericality_of :watchd_scan_period, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000

  validates_numericality_of :drop_detection_cool_off, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000

  validates_numericality_of :grow_max_parallel, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000
  validates_numericality_of :grow_target_rate, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000000
  validates_numericality_of :grow_tcp_port, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 65536
  validates_numericality_of :grow_udp_port, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 65536
  validates_numericality_of :grow_completion_timeout, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000
  validates_numericality_of :grow_memory, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 1000000
  validates_numericality_of :grow_chunk_size, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000

  validates_numericality_of :package_timeout, :allow_blank => true, :only_integer => true, :greater_than => 0, :less_than => 10000000

  validates_confirmation_of :content_protect_password, :if => :enable_content_protection

  validate :filters_not_blank

  def initialize(options = {})
    options = options.symbolize_keys

    options = format_filters(options)

    super(watchfolder_defaults.merge!(options))
  end

  def placeholder_password?(password)
    password == '*****'
  end

  def remote_credentials_present?
    unless remote_password.present? || local_ssh_key_path.present?
      errors.add_to_base('Credentials for remote node must be present')
    end
  end

  def filters_not_blank
      watchfolder_filters.each do |f|
        if f[:pattern].blank?
          errors.add_to_base('File filters can not be blank.')

          # Don't want to create duplicate errors
          break
        end
      end

      growing_file_filters.each do |f|
        if f[:pattern].blank?
          errors.add_to_base('Growing file filters can not be blank.')
          break
        end
      end

      package_filters.each do |f|
        if f[:pattern].blank?
          errors.add_to_base('Package filters can not be blank.')
          return
        end
      end
  end

  def format_filters(options)
    wf_filters = []
    if options[:watchfolder_filters].present?
      options[:watchfolder_filters].sort.each do |f|
        wf_filters << f[1]
      end
    end

    gf_filters = []
    if options[:growing_file_filters].present?
      options[:growing_file_filters].sort.each do |f|
        gf_filters << f[1]
      end
    end

    pkg_filters = []
    if options[:package_filters].present?
      options[:package_filters].sort.each do |f|
        pkg_filters << f[1]
      end
    end

    options[:watchfolder_filters] = wf_filters
    options[:growing_file_filters] = gf_filters
    options[:package_filters] = pkg_filters

    return options
  end

  def watchfolder_defaults
    transfer_defaults = ConsoleConfig::ConsoleDefault.transfer_defaults

    {
        :require_password => true,
        :max_retries => transfer_defaults[:max_retry_attempts],
        :retry_timeout => transfer_defaults[:retry_interval_seconds],
        :retry_period => transfer_defaults[:retry_interval_seconds],
        :policy => transfer_defaults[:priority_and_policy],
        :target_rate => transfer_defaults[:target_rate],
        :min_rate => transfer_defaults[:minimum_rate],

        :remote_authentication_method => "Password",

        :snapshot_creation_period => 5,
        :queue_threshold => 5,
        :connect_timeout => 10,
        :sample_period => 2,

        :cool_off => 10,
        :remote_tcp_port => 33001,
        :remote_nodeapi_port => 9092,
        :remote_udp_port => 33001,
        :resume => true,
        :overwrite => 'diff',
        :preserve_uid => false,
        :preserve_gid => false,
        :preserve_time => true,
        :preserve_access_time => true,
        :preserve_creation_time => true,
        :preserve_modification_time => true,
        :drop_detection_strategy => 'cool_off_only',
        :drop_detection_cool_off => 10,
        :grow_max_parallel => 8,
        :grow_target_rate => transfer_defaults[:target_rate],
        :grow_policy => transfer_defaults[:priority_and_policy],
        :grow_tcp_port => 22,
        :grow_udp_port => 33001,
        :grow_completion_timeout => 10,
        :grow_memory => 128,
        :grow_chunk_size => 128,
        :package_timeout => 10,
        :package_final_transfer => 'last_file_in_list',

        :watchfolder_filters => [],
        :growing_file_filters => [],
        :package_filters => [],

        :version => 0,
        :fingerprint => '',
        :keypath => '',
        :mode => 'default_mode',
        :order => 'Start',
        :local_watchfolder_type => "Push",
        :disable_watchfolder_type => false,

        :cookie => '',
        :datagram_size => nil,
        :grow_connect_timeout => 10,
        :grow_datagram_size => nil,
        :grow_force_send_after => 2,
        :grow_min_rate => transfer_defaults[:minimum_rate],
        :grow_overwrite => 'diff',
        :grow_preserve_access_time => true,
        :grow_preserve_creation_time => true,
        :grow_preserve_gid => false,
        :grow_preserve_modification_time => true,
        :grow_preserve_time => true,
        :grow_preserve_uid => false,
        :grow_read_block_size => nil,
        :grow_resume => true,
        :grow_rexmsg_size => nil,
        :grow_write_block_size => nil,
        :max_parallel => 5,
        :raw_options => "[]",
        :read_block_size => nil,
        :rexmsg_size => nil,
        :symbolic_links => 'follow',
        :synchronize_empty_directory => false,
        :tags => "{}",
        :watchd_connection => nil,
        :watchd_expire_in => 86400,
        :watchd_identifier => nil,
        :watchd_scan_period => 1800,
        :watchd_uri => nil,
        :write_block_size => nil
    }
  end

  # Creates Payload for Push WatchFolder for Node server 3.8 before that support only Old API - wf-version - 2017_09_14
  # @return [Array]
  def craft_payload
    options = {
        :meta => {
            :name => name,
            :version => version
        },
        :watchd => {
          :scan_period => "#{watchd_scan_period}s"
        },
        :local_watchfolder_type => local_watchfolder_type,
        :order => order,
        :mode => mode,
        :source_dir => local_path,
        :target_dir => remote_path,
        :cool_off => cool_off.to_s + "s",
        :filters => watchfolder_filters,
        :drop => {
            :detection_strategy => drop_detection_strategy,
            :cool_off => "#{drop_detection_cool_off}s"
        },
        :post_processing => {},
        :snapshot_creation_period => "#{snapshot_creation_period}s",
        :transport => {
            :host => remote_address,
            :user => remote_login,
            :fingerprint => fingerprint.nil? ? "" : fingerprint,
            :keypath => keypath.nil? ? "" : keypath,
            :token => token,
            :proxy => proxy,
            :error_handling => {
              :file => {
                  :max_retries => max_retries,
                  :retry_timeout => "#{retry_timeout.to_s}s"
              },
              :drop => {
                  :retry_period => "#{retry_period.to_s}s"
              }
            },
            :regular => {
                :max_parallel => max_parallel,
                :policy => policy,
                :min_rate => "#{min_rate.to_s}K",
                :target_rate => "#{target_rate.to_s}K",
                :tcp_port => remote_tcp_port,
                :udp_port => remote_udp_port,

                :queue_threshold => "#{queue_threshold}s",
                :connect_timeout => "#{connect_timeout}s",
                :sample_period => "#{sample_period}s",

                :cipher => cipher,
                :resume => resume ? resume_selection : 'none',
                :overwrite => overwrite.to_s.downcase,
                :preserve_uid => preserve_uid,
                :preserve_gid => preserve_gid,
                :preserve_time => preserve_time,
                :preserve_creation_time => preserve_creation_time,
                :preserve_access_time => preserve_access_time,
                :preserve_modification_time => preserve_modification_time,

                :datagram_size => datagram_size.nil? ? "" : datagram_size.to_s,
                :raw_options => JSON.parse(raw_options),
                :read_blk_size => read_block_size.nil? ? "" : read_block_size.to_s,
                :rexmsg_size => rexmsg_size.nil? ? "" : rexmsg_size.to_s,
                :symbolic_links => symbolic_links.to_s.downcase,
                :write_blk_size => write_block_size.nil? ? "" : write_block_size.to_s
            },
            :growing_file => {
                  :max_parallel => grow_max_parallel,
                  :policy => grow_policy,
                  :target_rate => "#{grow_target_rate}K",
                  :tcp_port => grow_tcp_port,
                  :udp_port => grow_udp_port,
                  :cipher => grow_cipher,
                  :completion_timeout => "#{grow_completion_timeout.to_s}s",
                  :memory => "#{grow_memory.to_s}M",
                  :chunk_size => "#{grow_chunk_size.to_s}K",
                  :filters => growing_file_filters,

                  :connect_timeout => "#{grow_connect_timeout}s",
                  :datagram_size => grow_datagram_size.nil? ? "" : grow_datagram_size.to_s,
                  :force_send_after => "#{grow_force_send_after}s",
                  :min_rate => "#{grow_min_rate.to_s}K",
                  :overwrite => grow_overwrite.to_s.downcase,
                  :preserve_access_time => grow_preserve_access_time,
                  :preserve_creation_time => grow_preserve_creation_time,
                  :preserve_gid => grow_preserve_gid,
                  :preserve_modification_time => grow_preserve_modification_time,
                  :preserve_time => grow_preserve_time,
                  :preserve_uid => grow_preserve_uid,
                  :read_blk_size => grow_read_block_size.nil? ? "" : grow_read_block_size.to_s,
                  :resume => grow_resume ? grow_resume_selection : 'none',
                  :rexmsg_size => grow_rexmsg_size.nil? ? "" : grow_rexmsg_size.to_s,
                  :write_blk_size => grow_write_block_size.nil? ? "" : grow_write_block_size.to_s
            }
        }
    }

    # Allow edited watchfolders to omit password
    # Password convention
    # - Not provided: use old password
    # - Provided but nil: clear password
    # - Else: update password
    options[:transport][:pass] = remote_password if remote_password.present? && !placeholder_password?(remote_password)

    options[:transport][:keypath] = local_ssh_key_path if local_ssh_key_path.present?

    options[:id] = uuid if uuid.present?


    # Post Processing settings
    if delete_after_transfer_per_drop
      options[:post_processing][:local] = {
          :type => "TRANSFER_DELETE",
          :archive_dir => ""
      }
    elsif delete_after_transfer_per_file
      options[:post_processing][:local] = {
          :type => "FILE_TRANSFER_DELETE",
          :archive_dir => ""
      }
    elsif archive_after_transfer
      options[:post_processing][:local] = {
          :type => "TRANSFER_ARCHIVE",
          :archive_dir => archive_dir
      }
    else
      options[:post_processing][:local] = {
          :type => "TRANSFER_NONE",
          :archive_dir => ""
      }
    end

    # Package filters
    if package_filters.present?
      options[:packages] = {
          :timeout => package_timeout.to_s + "s",
          :parsers => [
              {
                  :type => 'aspera_filelist',
                  :final_transfer => package_final_transfer,
                  :filters => package_filters
              }
          ]
      }
    else
      options[:packages] = {
          :parsers => []
      }
    end

    # EAR settings
    # Password convention
    # - Not provided: use old password
    # - Provided but nil: clear password
    # - Else: update password
    if enable_content_protection?
      # Only provide password if it's given and it is not a placeholder
      if content_protect_password.present? && !placeholder_password?(content_protect_password)
        options[:transport][:regular][:content_protect_password] = content_protect_password
      end
    else
      # Provide option with nil value to disable it
      options[:transport][:regular][:content_protect_password] = ''
    end

    options.to_json
  end

  def node_basic_auth(remote_authentication_method)
    if remote_authentication_method == "NODE_BASIC" || remote_authentication_method == "SHARES"
      return true
    end
  end

  # Creates Payload for Push WatchFolder in Node server 3.8 and above that do support New API - wf-version - 2017_10_23
  # @param [String] order - can be Start / Pause / Stop
  # @return [Array]
  def craft_payload_push(order)
    host = URI.parse(remote_address).host
    options_push = {
        :order => order,
        :mode => mode,
        :meta => {
            :name => name,
            :version => version
        },
        :source => {
            :path => local_path
        },
        :target => {
            :location => {
                :authentication => {
                    :fingerprint => !fingerprint.nil? ? fingerprint : "",
                    :type => node_basic_auth(remote_authentication_method) ? remote_authentication_method : "SSH",
                    :user => remote_login
                },
                :host => host.blank? ? remote_address : host,
                :port => node_basic_auth(remote_authentication_method) ? remote_nodeapi_port : remote_tcp_port,
                :type => "REMOTE"
            },
            :path => remote_path
        }
    }
    options_push[:id] = uuid if uuid.present?
    options_push[:target][:location][:authentication][:pass] = remote_password if remote_password.present?
    options_push[:target][:location][:authentication][:keypath] = local_ssh_key_path if local_ssh_key_path.present?
    return options_push
  end

  # Creates Payload for Pull WatchFolder in Node server 3.8 and above that do support New API - wf-version - 2017_10_23
  # @param [String] order - can be Start / Pause / Stop
  # @return [Array]
  def craft_payload_pull(order)
    host = URI.parse(remote_address).host
    options_pull = {
        :order => order,
        :mode => mode,
        :meta => {
            :name => name,
            :version => version
        },
        :source => {
            :location => {
                :authentication => {
                    :fingerprint => !fingerprint.nil? ? fingerprint : "",
                    :type => node_basic_auth(remote_authentication_method) ? remote_authentication_method : "SSH",
                    :user => remote_login
                },
                :host => host.blank? ? remote_address : host,
                :port => node_basic_auth(remote_authentication_method) ? remote_nodeapi_port : remote_tcp_port,
                :type => "REMOTE"
            },
            :path => remote_path
        },
        :target => {
            :path => local_path
        }
    }
    options_pull[:id] = uuid if uuid.present?
    options_pull[:source][:location][:authentication][:pass] = remote_password if remote_password.present?
    options_pull[:source][:location][:authentication][:keypath] = local_ssh_key_path if local_ssh_key_path.present?
    return options_pull
  end

  # Creates Optional Attributes Payload for Push & Pull WatchFolder for New API - wf-version - 2017_10_23
  # @return [Array]
  def craft_payload_optional
    options_optional = {
        :cool_off => cool_off.to_s + "s",

        :drop => {
            :detection_strategy => drop_detection_strategy,
            :cool_off => "#{drop_detection_cool_off}s"
        },
        :watchd => {
          :scan_period => "#{watchd_scan_period}s"
        },

        :filters => watchfolder_filters,
        :post_processing => {},
        :snapshot_creation_period => "#{snapshot_creation_period}s",

        :transport => {
            :error_handling => {
                :drop => {
                    :retry_period => "#{retry_period.to_s}s"
                },
                :file => {
                    :max_retries => max_retries,
                    :retry_timeout => "#{retry_timeout.to_s}s"
                }
            },
            :growing_file => {
                :chunk_size => "#{grow_chunk_size.to_s}K",
                :cipher => grow_cipher,
                :completion_timeout => "#{grow_completion_timeout.to_s}s",
                :filters => growing_file_filters,
                :max_parallel => grow_max_parallel,
                :memory => "#{grow_memory.to_s}M",
                :policy => grow_policy,
                :target_rate => "#{grow_target_rate}K",
                :tcp_port => grow_tcp_port,
                :udp_port => grow_udp_port,

                :connect_timeout => "#{grow_connect_timeout}s",
                :datagram_size => grow_datagram_size.nil? ? "" : grow_datagram_size.to_s,
                :force_send_after => "#{grow_force_send_after}s",
                :min_rate => "#{grow_min_rate.to_s}K",
                :overwrite => grow_overwrite.to_s.downcase,
                :preserve_access_time => grow_preserve_access_time,
                :preserve_creation_time => grow_preserve_creation_time,
                :preserve_gid => grow_preserve_gid,
                :preserve_modification_time => grow_preserve_modification_time,
                :preserve_time => grow_preserve_time,
                :preserve_uid => grow_preserve_uid,
                :read_blk_size => grow_read_block_size.nil? ? "" : grow_read_block_size.to_s,
                :resume => grow_resume ? grow_resume_selection : 'none',
                :rexmsg_size => grow_rexmsg_size.nil? ? "" : grow_rexmsg_size.to_s,
                :write_blk_size => grow_write_block_size.nil? ? "" : grow_write_block_size.to_s
            },
            :proxy => proxy,
            :regular => {
                :cipher => cipher,
                :connect_timeout => "#{connect_timeout}s",
                :max_parallel => max_parallel,
                :min_rate => "#{min_rate.to_s}K",
                :overwrite => overwrite.to_s.downcase,
                :policy => policy,
                :preserve_access_time => preserve_access_time,
                :preserve_creation_time => preserve_creation_time,
                :preserve_gid => preserve_gid,
                :preserve_modification_time => preserve_modification_time,
                :preserve_time => preserve_time,
                :preserve_uid => preserve_uid,
                :queue_threshold => "#{queue_threshold}s",
                :resume => resume ? resume_selection : 'none',
                :sample_period => "#{sample_period}s",
                :target_rate => "#{target_rate.to_s}K",
                :tcp_port => remote_tcp_port,
                :udp_port => remote_udp_port,

                :datagram_size => datagram_size.nil? ? "" : datagram_size.to_s,
                :raw_options => JSON.parse(raw_options),
                :read_blk_size => read_block_size.nil? ? "" : read_block_size.to_s,
                :rexmsg_size => rexmsg_size.nil? ? "" : rexmsg_size.to_s,
                :symbolic_links => symbolic_links.to_s.downcase,
                :write_blk_size => write_block_size.nil? ? "" : write_block_size.to_s
            },
            :token => token,
            :cookie => cookie,
            :tags => JSON.parse(tags)
        },
        :synchronize_empty_directory => synchronize_empty_directory
    }

    # Post Processing settings
    if delete_after_transfer_per_drop
      options_optional[:post_processing][:source] = {
          :type => "TRANSFER_DELETE",
          :archive_dir => ""
      }
    elsif delete_after_transfer_per_file
      options_optional[:post_processing][:source] = {
          :type => "FILE_TRANSFER_DELETE",
          :archive_dir => ""
      }
    elsif archive_after_transfer
      options_optional[:post_processing][:source] = {
          :type => "TRANSFER_ARCHIVE",
          :archive_dir => archive_dir
      }
    else
      options_optional[:post_processing][:source] = {
          :type => "TRANSFER_NONE",
          :archive_dir => ""
      }
    end

    # Package filters
    if package_filters.present?
      options_optional[:packages] = {
          :timeout => package_timeout.to_s + "s",
          :parsers => [
              {
                  :type => 'aspera_filelist',
                  :final_transfer => package_final_transfer,
                  :filters => package_filters
              }
          ]
      }
    else
      options_optional[:packages] = {
          :parsers => []
      }
    end

    # EAR settings
    # Password convention
    # - Not provided: use old password
    # - Provided but nil: clear password
    # - Else: update password
    if enable_content_protection?
      # Only provide password if it's given and it is not a placeholder
      if content_protect_password.present? && !placeholder_password?(content_protect_password)
        options_optional[:transport][:regular][:content_protect_password] = content_protect_password
      end
    else
      # Provide option with nil value to disable it
      options_optional[:transport][:regular][:content_protect_password] = ''
    end

    options_optional[:watchd][:expire_in] =  watchd_expire_in.to_s + "s"
    
    unless watchd_connection.nil? || watchd_connection.empty?
      options_optional[:watchd][:connection] = JSON.parse(watchd_connection)
    end
    unless watchd_identifier.nil? || watchd_identifier.empty?
      options_optional[:watchd][:identifier] = watchd_identifier
    end
    unless watchd_uri.nil? || watchd_uri.empty?
      options_optional[:watchd][:uri] = watchd_uri
    end

    return options_optional
  end

  # Creates Payload for Push WatchFolder in Node server 3.8 and above that do support New API - wf-version - 2017_10_23
  # @return [JSON]
  def craft_payload_create_push_wf
    options = (craft_payload_push('Start')).merge(craft_payload_optional)
    options.to_json
  end

  # Creates Payload for Pull WatchFolder in Node server 3.8 and above that do support New API - wf-version - 2017_10_23
  # @return [JSON]
  def craft_payload_create_pull_wf
    options = (craft_payload_pull('Start')).merge(craft_payload_optional)
    options.to_json
  end

  def byte_size_to_bytes(bs, base=1024, factor=1)
    # byte format: "[<number>T] [<number>G] [<number>M] [<number>K] [<number>[B]]
    bs = bs.downcase

    bytes = 0

    # Tera
    bs.match(/(\d+)t/)
    bytes += $1.to_i * (base*base*base*base)

    # Giga
    bs.match(/(\d+)g/)
    bytes += $1.to_i * (base*base*base)

    # Mega
    bs.match(/(\d+)m/)
    bytes += $1.to_i * (base*base)

    # Kilo
    bs.match(/(\d+)k/)
    bytes += $1.to_i * (base)

    # Byte
    bs.match(/(\d+)b/)
    bytes += $1.to_i

    bytes / factor
  end

  # Convert to kbps
  def bandwidth_to_kbps(bw)
    byte_size_to_bytes(bw, 1000, 1000)
  end

  def downcase_filters(filter_list)
    filter_list.map{ |f|
      f['type'] = f['type'].downcase
      f['rule'] = f['rule'].downcase
      f
    }
  end

  def parse_time(str, factor=1)
    str = str.downcase

    seconds = 0

    # Weeks
    str.match(/(\d+)w/)
    seconds += $1.to_i * (7 * 24 * 60 * 60)

    # Days
    str.match(/(\d+)d/)
    seconds += $1.to_i * (24 * 60 * 60)

    # Hours
    str.match(/(\d+)h/)
    seconds += $1.to_i * (60 * 60)

    # Minutes
    str.match(/(\d+)m/)
    seconds += $1.to_i * 60

    # Seconds
    str.match(/(\d+)s/)
    seconds += $1.to_i

    seconds / factor
  end

  # Fill in fields from JSON response
  def set_from_json_response(data)
    data = JSON.parse(data)

    transport = data['transport']
    regular = transport['regular']
    drop = data['drop']

    grow = transport['growing_file']
    package = data['packages']
    package_parsers = package['parsers']
    watchd = data['watchd']

    self.name = data['meta']['name']
    self.uuid = data['id']
    self.snapshot_creation_period = parse_time(data['snapshot_creation_period'])


    if data['source_dir']
      self.local_path = data['source_dir']
      post = data['post_processing']['local']
    else
      post = data['post_processing']['source']
      if data['source']['location']['type'] == "REMOTE"
        self.local_watchfolder_type = "Pull"
        self.local_path = data['target']['path']
      else
        self.local_watchfolder_type = "Push"
        self.local_path = data['source']['path']
      end
    end

    # Remote options

    if transport['host']
      self.remote_address = transport['host']
    elsif self.local_watchfolder_type == "Push"
      self.remote_address = data['target']['location']['host']
    elsif self.local_watchfolder_type == "Pull"
      self.remote_address = data['source']['location']['host']
    end

    if data['target_dir']
      self.remote_path = data['target_dir']
    elsif self.local_watchfolder_type == "Push"
      self.remote_path = data['target']['path']
    elsif self.local_watchfolder_type == "Pull"
      self.remote_path = data['source']['path']
    end

    if transport['user']
      self.remote_login = transport['user']
    elsif self.local_watchfolder_type == "Push"
      self.remote_login = data['target']['location']['authentication']['user']
    elsif self.local_watchfolder_type == "Pull"
      self.remote_login = data['source']['location']['authentication']['user']
    end

    if transport['pass']
      self.remote_password = transport['pass']
    elsif transport['keypath']
      self.local_ssh_key_path = transport['keypath']
    elsif self.local_watchfolder_type == "Push"
      self.remote_password = data['target']['location']['authentication']['pass']
      self.local_ssh_key_path = data['target']['location']['authentication']['keypath']
    elsif self.local_watchfolder_type == "Pull"
      self.remote_password = data['source']['location']['authentication']['pass']
      self.local_ssh_key_path = data['source']['location']['authentication']['keypath']
    end

    unless data['source_dir']
      if self.local_watchfolder_type == "Push"
        self.remote_authentication_method = data['target']['location']['authentication']['type']
      elsif self.local_watchfolder_type == "Pull"
        self.remote_authentication_method = data['source']['location']['authentication']['type']
      end
    end

    if data['source_dir'] || self.remote_authentication_method == "SSH"
      if self.local_ssh_key_path.present?
        self.remote_authentication_method = "SSH Key"
      elsif self.remote_password.present?
        self.remote_authentication_method = "Password"
      end
    end

    self.remote_tcp_port = regular['tcp_port']

    unless data['source_dir']
      if self.local_watchfolder_type == "Push"
        if self.node_basic_auth(remote_authentication_method)
          self.remote_nodeapi_port = data['target']['location']['port']
        else
          self.remote_tcp_port = data['target']['location']['port']
        end
      elsif self.local_watchfolder_type == "Pull"
        self.remote_nodeapi_port = data['source']['location']['port'] # only nodeapi for pull
      end
    end

    self.remote_udp_port = regular['udp_port']

    self.token = transport['token']
    self.proxy = transport['proxy']

    # Transfer Options
    self.target_rate = bandwidth_to_kbps(regular['target_rate'])
    self.min_rate = bandwidth_to_kbps(regular['min_rate'])
    self.policy = regular['policy'].to_s.downcase
    self.cipher = regular['cipher'].to_s.downcase
    self.cool_off = parse_time(data['cool_off'])

    if transport['error_handling']
      if transport['error_handling']['file']
        self.retry_timeout = parse_time(transport['error_handling']['file']['retry_timeout'])
        self.max_retries = transport['error_handling']['file']['max_retries'].to_i
      end

      if transport['error_handling']['drop']
        self.retry_period = parse_time(transport['error_handling']['drop']['retry_period'])
      end
    end

    # File Handling Options
    self.resume_selection = regular['resume'].to_s.downcase
    self.resume = self.resume_selection.to_s.downcase != 'none' ? true : false
    self.overwrite = regular['overwrite'].to_s.downcase
    self.preserve_uid = regular['preserve_uid']
    self.preserve_gid = regular['preserve_gid']
    self.preserve_time = regular['preserve_time']
    self.preserve_creation_time = regular['preserve_creation_time']
    self.preserve_modification_time = regular['preserve_modification_time']
    self.preserve_access_time = regular['preserve_access_time']

    self.queue_threshold = parse_time(regular['queue_threshold'])
    self.connect_timeout = parse_time(regular['connect_timeout'])
    self.sample_period = parse_time(regular['sample_period'])

    # The password sent back is garbage
    self.content_protect_password = regular['content_protect_password']
    self.content_protect_password_confirmation = regular['content_protect_password']
    self.enable_content_protection = regular['content_protect_password'].present?

    #self.temp_dir = post['remote']['temp_dir'] if post['remote']

    if post
      self.archive_after_transfer = post['type'].to_s.downcase == 'transfer_archive'
      self.archive_dir = post['archive_dir']
      self.delete_after_transfer_per_drop = post['type'].to_s.downcase == 'transfer_delete'
      self.delete_after_transfer_per_file = post['type'].to_s.downcase == 'file_transfer_delete'
    end

    self.watchfolder_filters = downcase_filters(data['filters'])

    self.drop_detection_strategy = drop['detection_strategy'].to_s.downcase
    self.drop_detection_cool_off = parse_time(drop['cool_off'])

    # Growing File Options
    self.grow_max_parallel = grow['max_parallel']
    self.grow_policy = grow['policy'].to_s.downcase
    self.grow_target_rate = bandwidth_to_kbps(grow['target_rate'])
    self.grow_tcp_port = grow['tcp_port']
    self.grow_udp_port = grow['udp_port']
    self.grow_cipher = grow['cipher'].to_s.downcase
    self.grow_completion_timeout = parse_time(grow['completion_timeout'])
    self.grow_memory = grow['memory'].to_i
    self.grow_chunk_size = grow['chunk_size'].to_i
    self.growing_file_filters = downcase_filters(grow['filters'])

    # Package Options - Assuming only one package
    self.package_timeout = parse_time(package['timeout']) if package['timeout'].present?

    if package_parsers.length > 0
      parser = package_parsers.first

      self.package_final_transfer = parser['final_transfer'].to_s.downcase

      self.package_filters = downcase_filters(parser['filters'])
    end

    # Misc
    self.version = data['meta']['version']
    self.fingerprint = transport['fingerprint']
    self.keypath = transport['keypath']
    self.mode = data['mode']
    self.order = data['order']

    if transport['cookie']
       self.cookie = transport['cookie']
    end
    unless regular['datagram_size'].nil? || regular['datagram_size'].empty?
      self.datagram_size = byte_size_to_bytes(regular['datagram_size'])
    end
    self.max_parallel = regular['max_parallel']
    if (data['synchronize_empty_directory'])
      self.synchronize_empty_directory = data['synchronize_empty_directory']
    end
    unless regular['raw_options'].nil? || regular['raw_options'].empty?
      self.raw_options = regular['raw_options'].to_json
    end
    unless regular['read_blk_size'].nil? || regular['read_blk_size'].empty?
      self.read_block_size = byte_size_to_bytes(regular['read_blk_size'])
    end
    unless regular['rexmsg_size'].nil? || regular['rexmsg_size'].empty?
      self.rexmsg_size = byte_size_to_bytes(regular['rexmsg_size'])
    end
    unless regular['symbolic_links'].nil? || regular['symbolic_links'].empty?
      self.symbolic_links = regular['symbolic_links'].to_s.downcase
    end
    unless transport['tags'].nil? || transport['tags'].empty?
      self.tags = transport['tags'].to_json
    end
    unless regular['write_blk_size'].nil? || regular['write_blk_size'].empty?
      self.write_block_size = byte_size_to_bytes(regular['write_blk_size'])
    end
    if watchd
      unless watchd['scan_period'].nil? || watchd['scan_period'].empty?
        self.watchd_scan_period = parse_time(watchd['scan_period'])
      end
      unless watchd['expire_in'].nil? || watchd['expire_in'].empty?
        self.watchd_expire_in = parse_time(watchd['expire_in'])
      end
      unless watchd['connection'].nil? || watchd['connection'].empty?
        self.watchd_connection = watchd['connection'].to_json
      end
      self.watchd_identifier = watchd['identifier']
      self.watchd_uri = watchd['uri']
    end
    unless grow['connect_timeout'].nil? || grow['connect_timeout'].empty?
      self.grow_connect_timeout = parse_time(grow['connect_timeout'])
    end
    unless grow['datagram_size'].nil? || grow['datagram_size'].empty?
      self.grow_datagram_size = byte_size_to_bytes(grow['datagram_size'])
    end
    unless grow['force_send_after'].nil? || grow['force_send_after'].empty?
      self.grow_force_send_after = parse_time(grow['force_send_after'])
    end
    unless grow['min_rate'].nil? || grow['min_rate'].empty?
      self.grow_min_rate = bandwidth_to_kbps(grow['min_rate'])
    end
    unless grow['overwrite'].nil? || grow['overwrite'].empty?
      self.grow_overwrite = grow['overwrite'].to_s.downcase
    end
    unless grow['preserve_access_time'].nil?
      self.grow_preserve_access_time = grow['preserve_access_time']
    end
    unless grow['preserve_creation_time'].nil?
      self.grow_preserve_creation_time = grow['preserve_creation_time']
    end
    unless grow['preserve_gid'].nil?
      self.grow_preserve_gid = grow['preserve_gid']
    end
    unless grow['preserve_modification_time'].nil?
      self.grow_preserve_modification_time = grow['preserve_modification_time']
    end
    unless grow['preserve_time'].nil?
      self.grow_preserve_time = grow['preserve_time']
    end
    unless grow['preserve_uid'].nil?
      self.grow_preserve_uid = grow['preserve_uid']
    end
    unless grow['read_blk_size'].nil? || grow['read_blk_size'].empty?
      self.grow_read_block_size = byte_size_to_bytes(grow['read_blk_size'])
    end
    unless grow['resume'].nil? || grow['resume'].empty?
      self.grow_resume_selection = grow['resume'].to_s.downcase
      self.grow_resume = self.grow_resume_selection.to_s.downcase != 'none' ? true : false
    end
    unless grow['rexmsg_size'].nil? || grow['rexmsg_size'].empty?
      self.grow_rexmsg_size = byte_size_to_bytes(grow['rexmsg_size'])
    end
    unless grow['write_blk_size'].nil? || grow['write_blk_size'].empty?
      self.grow_write_block_size = byte_size_to_bytes(grow['write_blk_size'])
    end
  end

end
