module DatabaseConfigEncryption

  ENCRYPT = "ENCRYPT"
  DECRYPT = "DECRYPT"

  def secret
    @secret ||= YAML.load_file(PATH_TO_SECRET)["secret"]
  end

  def encrypt(password)
    EzCrypto::Key.new(secret).encrypt64(password).strip
  end

  def decrypt(password)
    EzCrypto::Key.new(secret).decrypt64(password).to_s
  end

  def is_encrypted?(password)
    begin
      EzCrypto::Key.new(secret).decrypt64(password)
    rescue
      return false
    end

    true
  end

  def encrypt_database_passwords
    encrypt_decrypt(ENCRYPT)
  end

  def decrypt_database_passwords
    encrypt_decrypt(DECRYPT)
  end

  def database_rb_file_path
    RAILS_ROOT + (Aspera::OS.windows? ? "/../../Common Files/Aspera/Common" : "/../common") + "/mysql/database.rb.yml"
  end

  def encrypt_decrypt(func_type)
    database_configuration_file = Rails.configuration.database_configuration_file
    mysql_database_yml = YAML::load_file(database_rb_file_path)

    db_configuration = perform_encrypt_decrypt(func_type, YAML::load_file(database_configuration_file), 'password')

    if !mysql_database_yml.key?(:path_to_secret) # required when run for the first time after applying patch
      mysql_database_yml[:path_to_secret] = PATH_TO_SECRET
    end
    mysql_db_configuration = perform_encrypt_decrypt(func_type, {"default" => mysql_database_yml}, :password)

    if !db_configuration.nil?
      persist_configuration_change(db_configuration, database_configuration_file)
    end

    if !mysql_db_configuration.nil?
      persist_configuration_change(mysql_db_configuration["default"], database_rb_file_path)
    end
  end

  def perform_encrypt_decrypt(func_type, configuration, password_key)

    return if configuration.nil? || !(configuration.is_a? Hash)

    changed = false
    configuration.keys.each do |env|
      password = configuration[env][password_key]
      next if !password.present? || (func_type == ENCRYPT && is_encrypted?(password)) || (func_type == DECRYPT && !is_encrypted?(password))

      configuration[env][password_key] = plain_string(func_type == ENCRYPT ? encrypt(password) : decrypt(password))
      changed = true
    end

    return configuration if changed
  end

  def persist_configuration_change(configuration, file_path)
    File.open(file_path, 'w+') do |file|
      file.write(configuration.to_yaml)
    end
  end

  def plain_string(s)
    if s.instance_variable_defined?(:@_rails_html_safe)
      s.send(:remove_instance_variable, :@_rails_html_safe)
    end
    s
  end

end
