namespace :aspera do
  namespace :node do

    desc 'Update name of node (managed or unmanaged) with given address'
    task :update_node_name => :environment do
      raise 'NODE_ADDRESS is required' unless ENV['NODE_ADDRESS']
      raise 'NODE_NAME is required' unless ENV['NODE_NAME']

      if fasp_node = FaspNode.find_by_node_address(ENV['NODE_ADDRESS'])
        fasp_node.update_attributes!(:node_name => ENV['NODE_NAME'])
      elsif partner_node = PartnerNode.find_by_address(ENV['NODE_ADDRESS'])
        partner_node.update_attributes!(:name => ENV['NODE_NAME'])
      else
        raise "Node with address #{ENV['NODE_ADDRESS']} not found."
      end

      puts "Successfully updated node name to #{ENV['NODE_NAME']}"
    end

  end

  namespace :managed_node do

    desc 'Update credentials of managed  with given address'
    task :update_credentials => :environment do
      raise 'NODE_ADDRESS is required' unless ENV['NODE_ADDRESS']
      raise 'NODE_API_LOGIN is required' unless ENV['NODE_API_LOGIN']
      raise 'NODE_API_PASSWORD is required' unless ENV['NODE_API_PASSWORD']

      credential_attributes = {
        :node_api_username => ENV['NODE_API_LOGIN'],
        :node_api_password => ENV['NODE_API_PASSWORD']
      }
      fasp_node =  FaspNode.find_by_node_address(ENV['NODE_ADDRESS'])

      if fasp_node
        fasp_node.update_credentials!(credential_attributes)
      else
        raise "Node with address #{ENV['NODE_ADDRESS']} not found."
      end
      puts "Credentials updated to node to #{ENV['NODE_ADDRESS']}"
    end

    task :remove_credentials => :environment do
      raise 'NODE_ADDRESS is required' unless ENV['NODE_ADDRESS']

      fasp_node =  FaspNode.find_by_node_address(ENV['NODE_ADDRESS'])

      if fasp_node
        fasp_node.remove_credentials
        fasp_node.remove_node_api_credentials
      else
        raise "Node with address #{ENV['NODE_ADDRESS']} not found."
      end
      puts "Credentials removed to node to #{ENV['NODE_ADDRESS']}"
    end

    task :delete_managed_node => :environment do
      raise 'NODE_ADDRESS is required' unless ENV['NODE_ADDRESS']

      fasp_node =  FaspNode.find_by_node_address(ENV['NODE_ADDRESS'])

      if fasp_node
        fasp_node.destroy
      else
        raise "Node with address #{ENV['NODE_ADDRESS']} not found."
      end
      puts "Node with address #{ENV['NODE_ADDRESS']} deleted"
    end
  end

end
