namespace :aspera do
  namespace :unmanaged_node do
    task :create => :environment do
      pn = {}

      if ENV['ASPERA_NODE_ADDRESS'].blank?
        raise "Must specify node address (ASPERA_NODE_ADDRESS)."
        return
      else
        pn[:node_address] = ENV['ASPERA_NODE_ADDRESS']
      end
      
      if ENV['ASPERA_NODE_NAME'].blank?
        raise "Must specify node name. (ASPERA_NODE_NAME)."
        return
      else
        pn[:node_name] = ENV['ASPERA_NODE_NAME']
      end
      
      pn[:ssh_port] = ENV['ASPERA_NODE_SSH_PORT'] || 22

      @node = PartnerNode.new(pn)
      if @node.save
        puts "Unmanaged Node (#{@node.name}/#{@node.address}) successfully created."
        @node.reload
        exit(@node.id)
      else
        raise @node.errors.full_messages.to_s
      end
    end
  end

  namespace :managed_node do
    task :create => :environment do
      node_attributes = {}
      default_ssh_encryption = 'AES256-ctr'

      if ENV['ASPERA_NODE_ADDRESS'].blank?
        raise "Must specify node address (ASPERA_NODE_ADDRESS)."
        return
      elsif ENV['ASPERA_NODE_NAME'].blank?
        raise "Must specify node name. (ASPERA_NODE_NAME)."
        return
      elsif ENV['ASPERA_NODE_API_LOGIN'].blank?
        raise "Must specify node API login. (ASPERA_NODE_API_LOGIN)."
        return
      elsif ENV['ASPERA_NODE_API_PASSWORD'].blank?
        raise "Must specify node API password. (ASPERA_NODE_API_PASSWORD)."
        return
      end

      node_attributes[:node_address] = ENV['ASPERA_NODE_ADDRESS']
      node_attributes[:node_name] = ENV['ASPERA_NODE_NAME']
      node_attributes[:ssh_port] = ENV['ASPERA_NODE_SSH_PORT'] || 22
      node_attributes[:v3_node_daemon_port] = ENV['ASPERA_NODE_API_PORT'] || 9092
      node_attributes[:ssh_encryption] = ENV['ASPERA_NODE_SSH_ENCRYPTION'] || default_ssh_encryption

      FaspNode.transaction do
        @node = FaspNode.new(node_attributes)
        @node.save!

        credential_attributes = {
          :node_api_username => ENV['ASPERA_NODE_API_LOGIN'],
          :node_api_password => ENV['ASPERA_NODE_API_PASSWORD']
        }

        if ENV['ASPERA_SSH_LOGIN'].present?
          credential_attributes[:configuration_login] = ENV['ASPERA_SSH_LOGIN']
          credential_attributes[:configuration_password] = ENV['ASPERA_SSH_PASSWORD']
          credential_attributes[:configuration_ssh_key_id] = ENV['ASPERA_SSH_KEY_ID']
        end

        @node.update_credentials!(credential_attributes)

        puts "Managed Node (#{@node.id} #{@node.node_name}/#{@node.address}) successfully created."
      end
    end
  end
  
  namespace :endpoint do
    task :create => :environment do
      ep = {}

      if ENV['ASPERA_ENDPOINT_OWNER_ID'].blank?
        raise "Must set ASPERA_ENDPOINT_OWNER_ID"
      else 
        begin
          owner_id = Integer(ENV['ASPERA_ENDPOINT_OWNER_ID'])
          ep[:owner_id] = owner_id
        rescue Exception => e
          raise e.message
          return
        end
      end

      fn = PartnerNode.find(owner_id)

      ep[:address] = fn.address
      ep[:port] = ENV['ASPERA_ENDPOINT_SSH_PORT'] || fn.ssh_port
      ep[:login] = ENV['ASPERA_ENDPOINT_LOGIN']
      
      if ENV['ASPERA_ENDPOINT_SSH_KEY_ID'].present?
        begin
          ssh_key_id = Integer(ENV['ASPERA_ENDPOINT_SSH_KEY_ID'])
          ep[:ssh_key_id] = ssh_key_id
        rescue Exception => e
          raise e.message
          return
        end
      else
        if ENV['ASPERA_ENDPOINT_PASSWORD'].present?
          pw = ENV['ASPERA_ENDPOINT_PASSWORD']
          ep[:password] = pw
        end
      end
              
      endpoint = fn.endpoints.new(ep)
      if endpoint.save
        puts "Endpoint (#{endpoint.address}) has been saved."
        
        if ENV['ASPERA_ENDPOINT_EMAILS'].present?
          ENV['ASPERA_ENDPOINT_EMAILS'].split(',').each do |email_address|
            endpoint.email_notification_list_rows << LoginEmail.new({:email => email_address})
          end
        end
        
      else
        raise "Could not save endpoint (#{endpoint.address}) #{endpoint.errors.full_messages.to_s}"
      end
    end

  end
end
