const getConsoleApiEndpoint = (api_type, id) => {
  const CONSOLE_API_MAIN_ENDPOINT = '/aspera/console/api';
  switch (api_type){
    case CONSOLE_API_MAP.TRANSFER_PAUSE:
      return `${CONSOLE_API_MAIN_ENDPOINT}/transfers/${id}/pause`;
    case CONSOLE_API_MAP.TRANSFER_CANCEL:
      return `${CONSOLE_API_MAIN_ENDPOINT}/transfers/${id}/cancel`;
    case CONSOLE_API_MAP.TRANSFER_RESUME:
      return `${CONSOLE_API_MAIN_ENDPOINT}/transfers/${id}/resume`;
    case CONSOLE_API_MAP.TRANSFER_CHANGE:
      return `${CONSOLE_API_MAIN_ENDPOINT}/transfers/${id}/change_rate`;
    case CONSOLE_API_MAP.TRANSFER_CHANGE_POLICY:
        return `${CONSOLE_API_MAIN_ENDPOINT}/transfers/${id}/change_policy`;
    default:
      return `${CONSOLE_API_MAIN_ENDPOINT}/transfers/${id}`;
  }
}

const CONSOLE_API_MAP = {
  TRANSFER_PAUSE: 'pause',
  TRANSFER_CANCEL: 'cancel',
  TRANSFER_RESUME: 'resume',
  TRANSFER_CHANGE: 'set',
  TRANSFER_CHANGE_POLICY: 'policy',
}

const getKbFromMb = value => value * 1000;
const getMbFromKb = value => (value/1000).toFixed(2);

const control_transfer_for_node = async (action_type) => {
  const selected_transfer_DOM = document.getElementById('transfer-select');
  const transfer_id = selected_transfer_DOM.value;
  const transfer_display_name = selected_transfer_DOM.options[selected_transfer_DOM.selectedIndex].text;
  let options;

  switch(action_type){
    case CONSOLE_API_MAP.TRANSFER_PAUSE:
      result = await control_single_transfer('pause', transfer_id, transfer_display_name);
      break;
    case CONSOLE_API_MAP.TRANSFER_RESUME:
      result = await control_single_transfer('resume', transfer_id, transfer_display_name);
      break;
    case CONSOLE_API_MAP.TRANSFER_CHANGE:
      options = set_transfer_check_and_get_payload();
      result = options && await control_single_transfer('set', transfer_id, transfer_display_name, options);
      break;
    case CONSOLE_API_MAP.TRANSFER_CANCEL:
      if(!confirmCancelTransfer()){ return };
      result = await control_single_transfer('cancel', transfer_id, transfer_display_name);
      remove_transfer_from_select(transfer_id);
      break;
    case CONSOLE_API_MAP.TRANSFER_CHANGE_POLICY:
      options = set_policy_check_and_get_policy_payload();
      result = options && await control_single_transfer('policy', transfer_id, transfer_display_name, options);
      break;
    default:
      console.log('no action matched.');
      break;
  }
}

const control_transfer_for_session = async (action_type) => {
  const job_id = document.getElementById('job_id_hidden').value;
  let result;
  let options;

  switch(action_type){
    case CONSOLE_API_MAP.TRANSFER_PAUSE:
      result = await control_single_transfer('pause', job_id);
      break;
    case CONSOLE_API_MAP.TRANSFER_RESUME:
      result = await control_single_transfer('resume', job_id);
      break;
    case CONSOLE_API_MAP.TRANSFER_CHANGE:
      options = set_transfer_check_and_get_payload();
      result = options && await control_single_transfer('set', job_id, null, options);
      break;
    case CONSOLE_API_MAP.TRANSFER_CANCEL:
      if(!confirmCancelTransfer()){ return };
      result = await control_single_transfer('cancel', job_id);
      result && cancel_transfer_UI_change();
      break;
    case CONSOLE_API_MAP.TRANSFER_CHANGE_POLICY:
      options = set_policy_check_and_get_policy_payload();
      result = options && await control_single_transfer('policy', job_id, null, options);
      break;
    default:
      console.log('no action matched.');
      break;
  }
}

const control_single_transfer = (action_type, transfer_id, transfer_display_name=null, default_options=null) => {
  const options = default_options || {
    method: 'PUT',
    headers: getReqHeader(),
  }
  let apiEndpoint;
  let UI_change_func;
  let alert_msg;
  const get_alert_msg = alert_msg_keyword => `Transfer ${transfer_display_name ? transfer_display_name : ''} has been ${alert_msg_keyword}.`;

  switch(action_type){
    case CONSOLE_API_MAP.TRANSFER_PAUSE:
      apiEndpoint = getConsoleApiEndpoint(CONSOLE_API_MAP.TRANSFER_PAUSE, transfer_id);
      UI_change_func = () => pause_transfer_UI_change();
      alert_msg = get_alert_msg('paused');
      break;
    case CONSOLE_API_MAP.TRANSFER_RESUME:
      apiEndpoint = getConsoleApiEndpoint(CONSOLE_API_MAP.TRANSFER_RESUME, transfer_id);
      UI_change_func = () => resume_transfer_UI_change();
      alert_msg = get_alert_msg('resumed');
      break;
    case CONSOLE_API_MAP.TRANSFER_CHANGE:
      apiEndpoint = getConsoleApiEndpoint(CONSOLE_API_MAP.TRANSFER_CHANGE, transfer_id);
      alert_msg = `Set ${transfer_display_name ? transfer_display_name : ''} \n \n${options.body && `Target Rate: ${getMbFromKb(options.body.target_rate_kbps)} Mbps`} \n${options.body && `Minimum Rate: ${getMbFromKb(options.body.min_rate_kbps)} Mbps`}`;
      break;
    case CONSOLE_API_MAP.TRANSFER_CANCEL:
      apiEndpoint = getConsoleApiEndpoint(CONSOLE_API_MAP.TRANSFER_CANCEL, transfer_id);
      alert_msg = get_alert_msg('cancelled');
      break;
    case CONSOLE_API_MAP.TRANSFER_CHANGE_POLICY:
      apiEndpoint = getConsoleApiEndpoint(CONSOLE_API_MAP.TRANSFER_CHANGE_POLICY, transfer_id);
      alert_msg = `Change ${transfer_display_name ? transfer_display_name : ''} policy to ${options.body.policy_type_display}`;
      break;
    default:
        console.log('no action matched.');
        break;
    }
  if(options.body){
    options.body = JSON.stringify(options.body);
  }
  return fetch(apiEndpoint, options)
  .then(response => {
    alert(alert_msg);
    UI_change_func && UI_change_func();
    return response.status >= 200 && response.status < 300;
  })
  .catch(err => {console.log((err || {message: "Unknown error"}).message)})
}

const confirmCancelTransfer = () => confirm('Are you sure to cancel this transfer?') ? true : false;

const pause_transfer_UI_change = () => {
  jQuery(function($) {
    $('#pause_transfer').attr('disabled', '');
    $('#set_transfer').attr('disabled', '');
    $('#set_policy').attr('disabled', '');
    $('#resume_transfer').removeAttr('disabled');
  })
}

const resume_transfer_UI_change = () => {
  jQuery(function($) {
    $('#resume_transfer').attr('disabled', '');
    $('#pause_transfer').removeAttr('disabled');
    $('#set_transfer').removeAttr('disabled');
    $('#set_policy').removeAttr('disabled');
  })
}

const cancel_transfer_UI_change = () => window.location.replace('/aspera/console/dashboard');

const set_policy_check_and_get_policy_payload = () => {
  const policy_select_dom = document.getElementById('policy-select');
  const policy_type = policy_select_dom.value;
  const policy_type_display =  policy_select_dom.options[policy_select_dom.selectedIndex].text;
  if (policy_type === 'default'){
    alert('Please choose a policy');
    return null;
  }

  const options = {
    method: 'PUT',
    headers: getReqHeader(),
    body: {
      policy : policy_type,
      policy_type_display: policy_type_display,
    },
  }
  return options;
}

const set_transfer_check_and_get_payload = () => {
  //Both in Mbps as unit
  const target_rate = document.getElementById('target_rate').value;
  const min_rate = document.getElementById('min_rate').value;
  
  if (!target_rate && !min_rate){
    alert('Please set at least one of the rates.');
    return null;
  }

  if (target_rate === '' || min_rate === '') {
    alert('Please only enter number for rates.');
    return null;
  }

  if (parseInt(target_rate, 10) < 0 || parseInt(min_rate, 10) < 0){
    alert('Please set rate no smaller than 1.');
    return null;
  }

  if (parseInt(target_rate, 10) > 10000 || parseInt(min_rate, 10) > 10000){
    alert('Please set rate no larger than 10 GB/s.');
    return null;
  }

  const options = {
    method: 'PUT',
    headers: getReqHeader(),
    body: {
      min_rate_kbps : `${min_rate ? getKbFromMb(min_rate) : "0"}`,
      target_rate_kbps: `${target_rate ? getKbFromMb(target_rate) : "0"}`,
    },
  }
  return options;
}

const remove_transfer_from_select = transfer_id => {
  jQuery(function($) {
    $(`option[value="${transfer_id}"]`).remove();
    update_btn_with_selected_transfer();
  });
}

const capitalize = (s) => {
  if (typeof s !== 'string') return ''
  return s.charAt(0).toUpperCase() + s.slice(1)
}

const update_btn_with_selected_transfer = (transfer_id=undefined) => {
  jQuery(async function($) {
    let selectedTransferId = transfer_id || $('#transfer-select').val();
    selectedTransferId = isNaN(selectedTransferId) ? selectedTransferId : parseInt(selectedTransferId, 10);
    const selectedTransferDetail = await getTransferDetail(selectedTransferId);

    $('#policy-select').val(capitalize(selectedTransferDetail.policy));
    $('#target_rate').val(getMbFromKb(selectedTransferDetail.target_rate_kbps));
    $('#min_rate').val(getMbFromKb(selectedTransferDetail.min_rate_kbps));
    
    if ( $('#policy-select').length >= 1) {
      $('#policy-select').removeAttr('disabled');
    }
    if (selectedTransferDetail.status === 'paused'){
      $('#cancel_transfer').removeAttr('disabled');
      pause_transfer_UI_change();
    } else if (selectedTransferDetail.status === 'running'){
      $('#cancel_transfer').removeAttr('disabled');
      resume_transfer_UI_change();
    }
  })
}
