require File.dirname(File.expand_path(__FILE__))+"/ext/core"
require "yaml"
require "pp"
require "serializable"

class YmlLoadError < RuntimeError
  attr_accessor :file_name
end

class ObjectFactory
  class << self
    # jwe todo: include Enumerable and loop through @@components

    attr_accessor :components
    
    def class_name(file)
      File::basename(file).gsub(".rb","").camelize
    end
    
    def yml_file_name(file)
      File::dirname(File.expand_path(file))+"/#{File::basename(file).downcase}.yml"
    end
    
    def save( instance, path=nil )
      if instance.respond_to?(:dependencies) && !instance.dependencies.nil?
        instance.dependencies.each { |d|
          save( d )
        }
      end
      if !path && instance.is_serializable?()
        path = yml_file_name(instance.file_system_path)
        if path
          instance.save_config
          # log "Saving #{instance.class.name} to #{path}" 
          File.open(path, "w") do |file|
            file.write(instance.config.to_yaml({}))
          end
          unless Platform.windows?
            # MP need 640 for aspera_db_space_watcher to be able to pull credentials from database.rb.yml
            chmod(0640, path)
          end
        end
      end
    end

    def load_from_yml(c)
      yml_file = yml_file_name(c.class.file_system_path)
      if File.exists?(yml_file)
        c.load_config(YAML.load_file_with_retry(yml_file) || {})
      end
    end

    def reload_all
      @@components.each do |c|
        if c.class.is_serializable?
          load_from_yml(c)
        end
      end
    end

    def reload_one(klass)
      @@components.each do |c|
        if c.class.name == klass && c.class.is_serializable?
          load_from_yml(c)
          break
        end
      end
    end
    
    def save_all
      @@components ||= []
      @@components.each { |c|
        save( c )
      }
    end
    
    def load( klass, *args )
      @@components ||= []
      ret = @@components.find { |c| c.class==klass }
      return ret if ret != nil
      reload( klass, *args )
    end
    
    def reload( klass, *args )
      log "Instantiating klass #{klass}."
      
      instance = klass.new(*args)
      if klass.is_serializable?
        yml_file = yml_file_name(klass.file_system_path)
        if File.exists?(yml_file)
          begin
            yml_contents = YAML.load_file_with_retry(yml_file)
          rescue ArgumentError => exception
            if instance.respond_to? :corrupted_config
              yml_contents = instance.corrupted_config(yml_file,exception) 
            else
              e = YmlLoadError.new("Error loading config from #{yml_file}: #{$!}")
              e.file_name = yml_file
              raise e
            end
          end
          if yml_contents.class == klass
            instance = yml_contents
            instance.send(:load_config, instance.default_config || {})
          else
            instance.load_config(yml_contents || {})
          end
        end
      end
      @@components << instance
      instance.send( :post_initialize ) if instance.respond_to?( :post_initialize ) 
      instance      
    end
    
    def select
      @@components.select do |component|
        yield(component)
      end
    end
    alias :find_all :select 

    def available?(klass_name)
      if Object.const_defined?(klass_name)
        klass = eval(klass_name)
        ObjectFactory.load(klass)
      else
        log "Unavailable: #{klass_name} not defined."
        nil
      end
    rescue NameError
      log "Unavailable (NameError): #{klass_name}."
      nil
    end
    
  end # << self
  
end



